#include "Command.h"

#include "error.h"
Command::Command(ISocket* sock) : m_io(sock)
{

}

Command::Command()
{

}

Command::Command(const Command& sock) : m_io(sock.m_io)
{

}

const Command& Command::operator=(const Command& m)
{
	m_command=m.m_command;
	m_args=m.m_args;
	m_io=m_io;
	return m;
}

Command Command::parse(ISocket* s){
	return CommandParser::parse(s);
}

CommandParser::Token CommandParser::_set(Token t)
{
	m_token=t;
	switch (t) {
		case PV: m_current=";"; break;
		case INT: m_val=Json::Value(atoi(m_current.c_str())); break;
		case FLOAT: m_val=Json::Value(atof(m_current.c_str())); break;
		case STRING: m_val=Json::Value(m_current); break;
		case IDENT:
			if(m_current=="true"){
				m_val=Json::Value(true);
				m_token=BOOL;
			}
			else if(m_current=="false"){
				m_val=Json::Value(false);
				m_token=BOOL;
			}
			else{
				m_val=Json::Value(m_current);
			}
			break;
	}
	return t;
}

CommandParser::Token CommandParser::_next()
{

	m_current="";
	while(m_c==' ' || m_c=='\t' || m_c=='\r')_nc();
	if(m_c==';' || m_c=='\n'){
		return _set(PV);
	}
	else if(m_c=='"'){
		while(_nc()!='"')
			m_current+=m_c;
		_nc();
		_set(STRING);
	}
	else if(m_c=='\''){
		while(_nc()!='\'')
			m_current+=m_c;
		_nc();
		_set(STRING);
	}
	else if(m_c>='0' && m_c<='9')
	{
		m_current+=m_c;
		int dot=0;
		while(dot<=1 && ((_nc()>='0' && m_c<='9')||m_c=='.') )
			m_current+=m_c;
		if(dot>1){
			THROW(LexerError, "Un seul point ('.') est autorisé pour faire un nombre");
		}
		 _set(dot?FLOAT:INT);
	}
	else if( (m_c>='A' && m_c<='Z') ||(m_c>='a' && m_c<='z') || m_c=='_'){

		while((m_c>='A' && m_c<='Z') ||(m_c>='a' && m_c<='z') || m_c=='_'){
			m_current+=m_c;
			_nc();
		}
		_set(IDENT);
	}
	//while(m_c==' ' || m_c=='\t' || m_c=='\r') _nc();
	return m_token;

}


Command CommandParser::_parse()
{

	Command c(m_io);
	_nc();
	if(_next()!=PV){
		if(m_token!=IDENT){
			THROW(ParseError,"Erreur ident attendu en début de commande\n");
		}

		c.m_command=m_current;

		while(_next()!=PV){
			c.m_args.push_back(m_val);
		}
	}
	return c;
}



std::string Command::str() const{
	int s = m_args.size();
	std::string out;
	out+=m_command+"(";

	for(int i=0; i<s; i++){
		if(i) out+=", ";
		out+=m_args[i].asString();
	}
	out+=")";
}

void Command::print()
{
	m_io->write(str());
}
