#include "MidiMessage.h"
#include "error.h"

MidiMessage* MidiMessage::parse(const MidiRawMessage& m)
{
    uint8_t type = m[0];

    if(type<0xF0)
    {
        uint8_t ch = (type&0x0f)+1;
        type&=0xF0;
        switch(type){
            case MidiMessage::NOTE_ON:
                if(m.size()!=3)
					THROW(MidiParseException, "NoteOn need 3 bytes (found:"+str(m.size())+" in "+str(m)+")");
                return new NoteOn(ch, m[1], m[2]);
            case MidiMessage::NOTE_OFF:
                if(m.size()!=3)
					THROW(MidiParseException, "NoteOff need 3 bytes (found:"+str(m.size())+" in "+str(m)+")");
                return new NoteOff(ch, m[1], m[2]);
            case MidiMessage::CONTROLLER_CHANGE:
                return new ControllerChange(ch, m[1], m[2]);

            default:
                printf("%2X %2X %2X\n", m[0], m[1], m[2]);
				THROW(MidiParseException, "Unknown message "+str(m));
        }
	}
}


MidiMessage* MidiMessage::parse(Json::Value& v)
{
    if(!v.isArray() || !v.size()) return NULL;
    std::string _type = v[0].asString();
    std::string type;
    int s = _type.size();
    for(int i=0; i<s; i++) type+=tolower(_type[i]);

    if(type=="noteon" || type=="noteoff")
    {
		if(s<4) THROW(MidiParseException,"JsonErreur: NoteOn doit avoir 3 paramètre supplémentaires");
        Note* n = (type=="noteon")?
            ((Note*)new NoteOn(-1,-1,-1)):((Note*)new NoteOff(-1,-1,-1));
        if(!v[1].isNull()) n->channel=v[1].asInt();
        if(!v[2].isNull()) n->key=v[2].asInt();
        if(!v[3].isNull()) n->velocity=v[3].asInt();
        return n;
    }
	THROW(MidiParseException, "Uniqument le message NoteOn et NoteOff sont supportés pour MidiMessage::parse(Json::Value& v)");
    return NULL;
}
