#include "MidiPort.h"

#include "MidiMessage.h"

void __RtMidiInCallback(double ts, MidiRawMessage* rm, void *data)
{
    MidiPortIn* self = static_cast<MidiPortIn*>(data);
    self->_handler(ts, rm);
}

void __RtMidiErrorCallback(RtMidiError::Type type, const std::string &errorText, void *data)
{
    MidiPortBase* self = static_cast<MidiPortBase*>(data);
    self->_error_handler(type, errorText);
}



MidiPortIn::MidiPortIn(const std::string &clientName) :
    RtMidiIn(Api::LINUX_ALSA, clientName, 100),
    MidiPortBase()
{
    m_callback=NULL;
    m_callback_data=NULL;
    m_listener=NULL;
    setCallback(&__RtMidiInCallback, this);
    setErrorCallback(&__RtMidiErrorCallback, this);
}


void MidiPortIn::_handler(double ts, MidiRawMessage* rm)
{
    MidiMessage* m = MidiMessage::parse(*rm);
    if(m_listener)
    {
        m_listener->on_new_message(ts, m);
        delete m;
    }else if(m_callback)
    {
        m_callback(ts, m, m_callback_data);
        delete m;
    }else{
    }
}


MidiPortBase::MidiPortBase(){
	m_error_callback=NULL;
	m_error_callback_data=NULL;
	m_error_listener=NULL;
}

void MidiPortBase::_error_handler(RtMidiError::Type type, const std::string &errorText)
{
    if(m_error_listener)
    {
        m_error_listener->on_error(type, errorText);
    }else if(m_error_callback)
    {
        m_error_callback(type, errorText, m_error_callback_data);
    }else{
    }
}


MidiPortOut::MidiPortOut(const std::string &clientName) :
        RtMidiOut(Api::LINUX_ALSA, clientName), MidiPortBase()
{
    setErrorCallback(&__RtMidiErrorCallback, this);
}


void MidiPortOut::send(const MidiMessage& m)
{
    uint8_t buffer[3];
    int n = m.get_bytes(buffer);
    sendMessage(buffer, n);
}



void MidiPortOut::send(const MidiMessageList& m)
{
    uint8_t buffer[3];
    int s = m.size();
    for(int i=0; i<s; i++)
    {
        int n = m[i]->get_bytes(buffer);
        sendMessage(buffer, n);
    }
}

std::vector<std::string> MidiPortIn::get_ports()
{
	std::vector<std::string> ret;
	RtMidiIn in(RtMidiIn::Api::LINUX_ALSA, "--", 100);
	int s = in.getPortCount();
	for(int i=0; i<s; i++)
		ret.push_back(in.getPortName(i));
	return ret;
}


std::vector<std::string> MidiPortOut::get_ports()
{
	std::vector<std::string> ret;
	RtMidiIn out(RtMidiIn::Api::LINUX_ALSA, "--", 100);
	int s = out.getPortCount();
	for(int i=0; i<s; i++)
		ret.push_back(out.getPortName(i));
	return ret;
}
