#include "Operations.h"
#include "PadSelection.h"



#include "MidiMessage.h"

InputOperation* InputOperation::from_json(Json::Value & v)
{
    if(v["type"].asString()=="TRANSLATE")
        return new TranslationOperation(v);
    else if(v["type"].asString()=="DROP")
        return  new DropOperation();
    else if(v["type"].asString()=="RANGE")
        return  new RangeOperation(v);
    else if(v["type"].asString()=="REORDER")
        return  new ReorderOperation(v);

    return NULL;
}

TranslationOperation::Mode TranslationOperation::parse_mode(const std::string& v)
{
    if(v=="RELATIVE") return TranslationOperation::RELATIVE;
    if(v=="UNCHANGED") return TranslationOperation::UNCHANGED;
    if(v=="FIXED") return TranslationOperation::FIXED;
    return TranslationOperation::RELATIVE;
}

TranslationOperation::TranslationOperation(Json::Value& v) : InputOperation(TRANSLATION)
{
    set(v["values"][0].asInt(), v["values"][1].asInt(), v["values"][2].asInt());
    set_mode(TranslationOperation::parse_mode(v["modes"][0].asString()),
            TranslationOperation::parse_mode(v["modes"][1].asString()),
            TranslationOperation::parse_mode(v["modes"][2].asString()));

}


MidiMessage* TranslationOperation::execute(PadSelection* sel, MidiMessage* m, int i) const
{
    Note* n=NULL;
    switch (m->type) {
        case MidiMessage::NOTE_OFF:
        case MidiMessage::NOTE_ON:
        case MidiMessage::CONTROLLER_CHANGE:
            n=dynamic_cast<Note*>(m);
            break;
        default:
            return m;
    }

    if(m_channel_mode==RELATIVE) n->channel+=m_channel;
    else n->channel=m_channel;

    if(m_key_mode==RELATIVE) n->key+=m_key;
    else n->key=m_key;

    if(m_velocity_mode==RELATIVE) n->velocity+=m_velocity;
    else n->velocity=m_velocity;

    return m;
}

MidiMessage* RangeOperation::execute(PadSelection* sel, MidiMessage* m, int i) const
{
    Note* n=NULL;
    switch (m->type) {
        case MidiMessage::NOTE_OFF:
        case MidiMessage::NOTE_ON:
        case MidiMessage::CONTROLLER_CHANGE:
            n=dynamic_cast<Note*>(m);
            break;
        default:
            return m;
    }

    int ygamme = (i)/m_range.size();
    int xgamme = (i)%m_range.size();
    fprintf(stderr, " %d -> %d\n",n->key, m_key+ygamme*m_length+m_range[xgamme]);
    n->key =m_key+ygamme*m_length+m_range[xgamme];
    return n;
}

ReorderOperation::ReorderOperation(Json::Value& v) : InputOperation(REORDER)
{
    m_offset=v["offset"].asInt();
}

MidiMessage* ReorderOperation::execute(PadSelection* sel, MidiMessage* m, int i) const
{
    Note* n=NULL;
    switch (m->type) {
        case MidiMessage::NOTE_OFF:
        case MidiMessage::NOTE_ON:
        case MidiMessage::CONTROLLER_CHANGE:
            n=dynamic_cast<Note*>(m);
            break;
        default:
            return m;
    }
    n->key=i;
    return n;
}
