#include "Pad.h"
#include "Config.h"
#include "PadManager.h"
#include "unistd.h"
#include "PadConfiguration.h"
#include "PadDefinition.h"


Pad::Pad()
{
   m_configuration=NULL;
   m_pad_definition=NULL;
}

Pad::Pad(const std::string& file)
{
	m_configuration=NULL;
	m_pad_definition=NULL;
	if(!load_configuration(file)){
		THROW(UnknownError, "Erreur dans le chargement du Pad");
	}
}

Pad::Pad(PadDefinition* p)
{
	m_configuration=NULL;
	m_pad_definition=p;
	m_name=p->get_name();
}

Pad::Pad(PadConfiguration* p)
{
	m_configuration=p;
	m_pad_definition=&p->get_pad_definition();
	m_name=m_pad_definition->get_name();
}

Pad::~Pad()
{
	if(m_configuration) delete m_configuration;
}

void Pad::remove_configuration()
{
    delete m_configuration;
    m_configuration=NULL;
}

bool Pad::load_configuration(const std::string& file)
{
    if(m_configuration){
        remove_configuration();
    }
    m_configuration = PadConfiguration::from_file(file);
    if(!m_configuration){
		THROW(UnknownError,"Impossible de charger le pad...");
    }
	PadDefinition * newpad = &m_configuration->get_pad_definition();

	if(m_pad_definition && m_pad_definition!=newpad){
		THROW(UnknownError,"Impossible de charger le pad...");
    }
	m_pad_definition = newpad;

    if(Config::get_bool("autoconnect")){
		connect(
				!m_pad_definition->get_input_port().isPortOpen(),
				!m_pad_definition->get_control_port().isPortOpen());
    }

    if(Config::get_bool("clear")){
		m_pad_definition->clear();
        //usleep(100000);
    }
    m_configuration->send_init();
    return true;
}

void Pad::connect(bool in, bool out)
{
	m_pad_definition->connect_to_device(in, out);
}


const std::string& Pad::get_name() const
{
	return m_name;
}


void Pad::clear()
{
	m_pad_definition->clear();
}
void Pad::input(MidiMessageList* m)
{
	input(m);
}

void Pad::input(Json::Value& msg){
	MidiMessageList* m = MidiMessageList::parse(msg);
	input(m);
	m->free_content();
}

void Pad::input(const std::string& msg){
	MidiMessageList* m = MidiMessageList::parse(msg);
	input(m);
	m->free_content();
}


