#include <unistd.h>
#include "PadConfiguration.h"
#include "PadSelection.h"
#include <fstream>
#include "PadDefinition.h"
#include "PadManager.h"
/**

    TRANSLATE = {
        "type" : "TRANSLATE",
        "values" : (int, int, int) # les valeur des channels, key et velocity
        "modes" : (str, str, str) # les modes des channels, key et velocity parmi ("RELATIVE" ou "FIXED")
    }

    DROP = {
        type : "DROP"
    }

    REORDER = {
        "type" : "REORDER",
        "offset" : int
    }

    RANGE = {
        type : "RANGE",
        "range" : [ 0,1 ,2,3] #les note %12
        "key" : int # note midi de départ
        "length" : longeuer de la gamme (souvent 12
    }

    SELECTION = {
        "operations" : [
            {TRANSLATE | DROP | RANGE}
        ],
        "init": [
            action,
        ],
        "locked" ; [ (x,y) ]
    }



    {
        "name" : string
        "description" string
        "methods" : {
            "clear" : INPUT_ACTION
        }
        "pad" : string # Nom du pad sur laquelle la configuration peut fonctionner
        "selections" : [
            {SELECTION}
        ],
        "matrix" : [string] # le nom de la sélection correspondant à chaque bouton
    }
*/

PadConfiguration::PadConfiguration(Json::Value& v) :
	_m_pad_definition_ptr(PadManager::get_pad(v["pad"].asString())),
	m_pad_definition(*_m_pad_definition_ptr),
	m_matrix(m_pad_definition.m_matrix)
{
	m_pad_definition.set_error_listener(this);
	m_pad_definition.set_port_listener(this);
	m_iport=m_pad_definition.m_iport;
	m_cport=m_pad_definition.m_cport;
	m_oport=m_pad_definition.m_oport;
	m_width=m_pad_definition.m_width;
	m_height=m_pad_definition.m_height;
	m_matrix=m_pad_definition.m_matrix;
    m_name=v["name"].asString();
    m_description=v["description"].asString();
    Json::Value sel = v["selections"], mat=v["matrix"];
    int i, s = sel.size();

    for( Json::Value::const_iterator itr = sel.begin() ; itr != sel.end() ; itr++ ) {
        std::string k = itr.key().asString();
        Json::Value v = *itr;
        m_selections.push_back(new PadSelection(k, v, this));
    }

    for(int i=0; i<m_width*m_height; i++)
    {
        if(mat[i].isString())
        {
            PadSelection* sel = selection_by_name(mat[i].asString());
            if(m_matrix[i])sel->add(m_matrix[i]);
        }
    }
}

PadDefinition& PadConfiguration::get_pad_definition()
{
	return m_pad_definition;
}



void PadConfiguration::send_init()
{
    usleep(10000);
    int s=m_selections.size();
    for(int i=0; i<s; i++)
        m_selections[i]->send_init();
}

PadConfiguration::~PadConfiguration()
{
    int s = m_selections.size();
    for(int i=0; i<s; i++)
        delete m_selections[i];
}


void PadConfiguration::on_error(RtMidiError::Type type, const std::string &errorText)
{
    std::cerr << "Midi Erreur : " << errorText << "\n";
}


void PadConfiguration::on_new_message(double ts, MidiMessage *m)
{
	AbsInput* in = m_pad_definition.get_by_message(m);
    PadSelection* sel = in->get_selection();
    if(sel)
    {
        m=sel->execute(in, m);
        if(m)m_oport->send(m);
    }else{
        m_oport->send(m);
    }
}

PadSelection* PadConfiguration::selection_by_name(const std::string& n)
{
    int s = m_selections.size();
    for(int i=0; i<s; i++)
        if(m_selections[i]->get_name()==n)
            return m_selections[i];
   return NULL;
}

PadConfiguration* PadConfiguration::from_file(const std::string& str)
{
    std::ifstream  fd(str);
    if(!fd.is_open()){
		THROW(NotFoundError, "Le fichier de configuration de pad '"<< str << "' est inaccessible");
    }

    std::string x;
    Json::Value val;
    try{
        fd >> val;
    }catch(...){
		THROW(ParseError, "Le fichier de configuration de pad '"<< str << "' est malformé (json invaide)");
    }
    fd.close();
    return new PadConfiguration(val);
}
