#include "PadManager.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
PadManager* PadManager::m_instance=NULL;
#include <fstream>
#include "Config.h"
#include "PadDefinition.h"

PadEntry::~PadEntry()
{
	if(instance) delete instance;
}

void PadManager::_init()
{
    std::string dir = Config::PAD_DIR();
    DIR* dirp = opendir(dir.c_str());
    struct dirent * dp;
    while ((dp = readdir(dirp)) != NULL) {
        std::string path=Config::PAD_DIR(dp->d_name);
        struct stat spath;

        stat(path.c_str(), &spath);
        try{
            std::ifstream fd(path);
            if(fd.is_open())
            {
                Json::Value v;
                fd >> v;
				if(v.isObject()){
					std::string name, in, out;
					if(v.isMember("name") && v["name"].isString())
						name=v["name"].asString();
					if(v.isMember("ports") && v["ports"].isObject()){
						Json::Value ports = v["ports"];
						if(ports.isMember("input") && ports["input"].isString())
							in=ports["input"].asString();
						if(ports.isMember("output") && ports["output"].isString())
							out=ports["output"].asString();
					}
					if(name!="" && in!="" && out!=""){
						PadEntry& p = *_add();
						p.name=name;
						p.in_port=in;
						p.out_port;
						p.filename=dp->d_name;
					}else{
						std::cerr << "Attention pad '"<< name <<"' ("<< path <<") est malformé\n";
					}
				}
                fd.close();
            }
        }catch(...){

        }
    }
    closedir(dirp);
}

int PadManager::_get_pad_index(const std::string& name)
{
	int i=0, l=m_list.size();
    for(i=0; i<l; i++)
		if(name==m_list[i]->name)
            return i;
    return -1;
}

PadDefinition* PadManager::_get_pad(const std::string& name)
{
    int i = _get_pad_index(name);
	if(!m_list[i]->instance)
    {
		PadDefinition* p = new PadDefinition(_read_pad(name));
		m_list[i]->instance=p;
    }
	return m_list[i]->instance;
}

bool PadManager::_has_pad(const std::string& pad_name) const
{
    int s=m_list.size();
    for(int i=0; i<s; i++)
		if(m_list[i]->name==pad_name)
           return true;
    return false;
}

Json::Value PadManager::_read_pad(const std::string &pad_name) const
{
    std::string filename;
    int s=m_list.size();
    for(int i=0; i<s; i++){
		if(m_list[i]->name==pad_name)
        {
		   filename=m_list[i]->filename;
           break;
        }
    }
    if(filename==""){
		THROW(NotFoundError, "Impossible de trouver le pad '" << pad_name << "'");
    }
    filename=Config::PAD_DIR(filename);
    std::ifstream fd(filename);
    if(fd.is_open()){
        try {
            Json::Value v;
            fd >> v;
            fd.close();
            return v;
        }  catch (...) {
            fd.close();
            std::cerr << "Erreur le fichier du pad '"<<pad_name<<"' ('"<<filename<<"')\n";
            return Json::Value();
        }

    }else{
        std::cerr << "Impossible de lire le fichier du pad '"<<pad_name<<"' ('"<<filename<<"')\n";
        return Json::Value();
    }
}

PadEntry* PadManager::_add()
{
	static int _max_id=-1;
	PadEntry* p = new PadEntry(++_max_id);
	m_list.push_back(p);
	return p;
}

void PadManager::_clean()
{
	int i, s=m_list.size();
	for(i=0; i<s; i++){
		delete m_list[i];
	}
	m_list.clear();
}
