#include "Socket.h"
#include <iostream>
#include <cstring>
#include "utils.h"
#include <cstdlib>
#include <cstdio>
#include "Command.h"
#include "errno.h"
_AbsSocket::_AbsSocket(SocketType type)
{
    m_type=type;
    m_is_open=false;
    m_sock = socket(AF_INET, SOCK_STREAM, 0);
    if(m_sock == INVALID_SOCKET)
	{
		THROW(SocketError, "_AbsSocket::_AbsSocket() socket creation " << strerror(errno));
    }
}

_AbsSocket::~_AbsSocket()
{
   close();
}

void _AbsSocket::close()
{
	if(m_is_open)
    {
        ::close(m_sock);
        m_is_open=false;
    }
}

bool _AbsSocket::is_open(){
	int error_code;
	socklen_t error_code_size = sizeof(error_code);
	getsockopt(m_sock, SOL_SOCKET, SO_ERROR, &error_code, &error_code_size);
	return error_code==0;
}

std::string& ISocket::readline(std::string & out)
{
    int c;
    while((c=readc())!='\n' && c>=0);
    return out;
}

void Socket::write(const void* data, int size)
{
    int i=0;
	while(size>0){
        int ret = ::send(m_sock, data, size, 0);
        if(ret < 0)
		{
			THROW(SendSocketError, "Socket::write(const void*, int)" << strerror(errno));
        }
        size-=ret;
    }
}

int Socket::read(void* data, int n, bool blocking)
{
    int readed = 0;
	while( readed<n){
        int ret;
        if((ret = recv(m_sock, data, n, 0)) < 0)
		{
			THROW(RecieveSocketError, "Socket::read(void*, int, bool)" << strerror(errno));
		}
        readed+=ret;
		if(readed<n && !blocking) break;
    }
    return readed;
}

bool InetSocket::connect(const std::string& addr, int port)
{
    struct hostent *hostinfo = NULL;
    SOCKADDR_IN sin = { 0 }; /* initialise la structure avec des 0 */


    hostinfo = gethostbyname(addr.c_str()); /* on récupère les informations de l'hôte auquel on veut se connecter */
    if (hostinfo == NULL) /* l'hôte n'existe pas */
	{
		THROW(HostSocketError, "InetSocket::connect(const std::string&, int) : hostname error" << strerror(errno));
    }

    sin.sin_addr = *(IN_ADDR *) hostinfo->h_addr; /* l'adresse se trouve dans le champ h_addr de la structure hostinfo */
    sin.sin_port = htons(port); /* on utilise htons pour le port */
    sin.sin_family = AF_INET;

    if(::connect(m_sock,(SOCKADDR *) &sin, sizeof(SOCKADDR)) == SOCKET_ERROR)
	{
		THROW(ConnectSocketError, "InetSocket::connect(const std::string&, int)" << strerror(errno));
    }
    m_is_open=true;
    return true;
}

InetServer::InetServer() : _AbsSocket(INET)
{
	m_sock = socket(AF_INET, SOCK_STREAM, 0);
    if(m_sock == INVALID_SOCKET)
	{
		THROW(SocketError, "InetSocket::InetServer() : créatiopn de socket" << strerror(errno));
    }
}

ISocket* InetServer::accept()
{
    SOCKADDR_IN csin = { 0 };
    SOCKET csock;
    int sinsize = sizeof csin;

    csock = ::accept(m_sock, (SOCKADDR *)&csin, (socklen_t*)&sinsize);

    if(csock == INVALID_SOCKET)
	{
		THROW(SocketError, "InetServer::accept()" << strerror(errno));
    }
	return new InetSocket(csock);
}

int _getc()
{
	int x;
	x=getc(stdin);
	printf("%d\n", x);
	return x;
}

ISocket* StdioServer::accept(){
	int x = 0;
	for(x=getc(stdin); x<0; x=getc(stdin))
		usleep(100000);

	return new StdioSocket((char)x);
}

void InetServer::listen(int port)
{
    SOCKADDR_IN sin = { 0 };
	int yes = 1;
	int ret;
    sin.sin_addr.s_addr = htonl(INADDR_ANY); /* nous sommes un serveur, nous acceptons n'importe quelle adresse */

    sin.sin_family = AF_INET;

    sin.sin_port = htons(port);

	if ( ret = setsockopt(m_sock, SOL_SOCKET, SO_REUSEADDR | SO_REUSEPORT, &yes, sizeof(yes))) {
		THROW(OptSocketError, "InetSocket::listen()" << strerror(errno));
	}

	if(ret = bind (m_sock, (SOCKADDR *) &sin, sizeof sin))
    {
		THROW(BindSocketError, "InetSocket::listen()" << strerror(errno));
	}
	if(ret = ::listen(m_sock, 5))
	{
		THROW(ListenSocketError, "InetSocket::listen()" << strerror(errno));
    }
}

bool UnixSocket::connect(const std::string& path)
{
	struct sockaddr_un addr;
	memset(&addr, 0, sizeof(struct sockaddr_un));
	addr . sun_family = AF_UNIX ;
	strncpy(addr.sun_path, path.c_str(), sizeof(addr.sun_path)-1);
	if(::connect(m_sock, (struct sockaddr *)&addr, sizeof(struct sockaddr_un)))
	{
		THROW(ConnectSocketError, "UnixSocket::connect(const std::string&)" << strerror(errno));
	}
	return true;

}

UnixServer::UnixServer() : _AbsSocket(UNIX)
{

    m_sock = socket(AF_UNIX, SOCK_STREAM, 0);
    if(m_sock == INVALID_SOCKET)
	{
		THROW(SocketError, "UnixServer::UnixServer()" << strerror(errno));
    }
}

ISocket* UnixServer::accept()
{
    struct sockaddr_un addr;
    SOCKET csock;
    memset(&addr , 0 , sizeof ( struct sockaddr_un ) ) ;
    int sinsize = sizeof addr;

    csock = ::accept(m_sock, (SOCKADDR *)&addr, (socklen_t*)&addr);

    if(csock == INVALID_SOCKET)
	{
		THROW(AcceptSocketError, "UnixSocket::accept()" << strerror(errno));
    }
    return new Socket(m_type, csock);
}

void UnixServer::listen(const std::string& path, bool force)
{
    struct sockaddr_un addr;
	memset(&addr, 0, sizeof(struct sockaddr_un ));
    addr.sun_family = AF_UNIX ;
    strncpy( addr.sun_path, path.c_str(), sizeof(addr.sun_path));

    m_path=path;
    if(Fs::is_dir(m_path))
	{
		THROW(ListenSocketError, "UnixSocket::listen(const std::string&, bool)" << strerror(errno));
    }
    if(Fs::is_file(m_path) && force)
        Fs::remove(m_path);
    if(bind (m_sock, (SOCKADDR *) &addr, sizeof addr) == SOCKET_ERROR)
	{
		THROW(BindSocketError, "UnixSocket::listen(const std::string&, bool)" << strerror(errno));
    }
    if(::listen(m_sock, 5) == SOCKET_ERROR)
    {
		THROW(ListenSocketError, "UnixSocket::listen(const std::string&, bool)" << strerror(errno));
    }
}

UnixServer::~UnixServer()
{
    close();
    Fs::remove(m_path);
}


void StdioSocket::write(const void* data, int size)
{
	fwrite(data, 1, size, stderr);
}


int  StdioSocket::read(void* data, int n, bool blocking)
{
	int m=0;
	if(m_first>=0)
	{
		((char*)data)[0]=(char)m_first;
		m_first=-1;
		m++;
	}
	return fread(data, 1, n-m, stdin)+m;
}
