#include "utils.h"
#include <fstream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

std::string str(int x, const char* format)
{
    char tmp[128];
    sprintf(tmp, format, x);
    return tmp;
}


std::string str(const MidiRawMessage& x)
{
    std::string out = "[";
    int size = x.size();
    for(int i=0; i<size; i++)
        out+=(i>0?" ":"")+str((int)x[i], "%2X");
    out+="]";
    return out;
}

template<typename T>
class RingBuffer
{
    public:
        RingBuffer(int n){
            m_data.resize(n);
            m_size=n;
            m_head=0;
            m_tail=-1;
        }
        virtual ~RingBuffer(){}

        void enqueue(const T& val){
            if( m_head==m_tail ) throw "Ring Buffer Overflow";
            m_data[m_head]=val;
            m_head=(m_head+1)%m_size;
        }

        T& dequeue(){
            if( ((m_tail+1)%m_size) != m_head)
            {
                m_tail=(m_tail+1)%m_size;
                return m_data[m_tail];
            }
            throw "Empty Ring Buffer";
        }

        T& peak() const
        {
            if( ((m_tail+1)%m_size) != m_head)
            {
                return m_data[(m_tail+1)%m_size];
            }
            throw "Empty Ring Buffer";
        }

        int size() const
        {
            if(m_head>m_tail)
            {
                return m_head-m_tail-1;
            }
            else{
                return m_size-m_tail-1+m_head;
            }
        }

    protected:
        int             m_size;
        int             m_head;
        int             m_tail;
        std::vector<T>  m_data;
};


bool start_with(const std::string& s1, const std::string& s2, bool ignoreCase)
{
    int ss1 = s1.size(), ss2=s2.size();
    int min=(ss1>ss2)?ss2:ss1;
    for(int i=0; i<min; i++){
        char c1 = s1[i], c2 = s2[i];
        if(ignoreCase){
            c1=tolower(c1);
            c2=tolower(c2);
        }
        if(c1!=c2) return false;
    }
    return true;
}

Json::Value json_parse_file(const std::string& filename)
{
    Json::Value root;   // will contains the root value after parsing.
    std::ifstream file(filename);
    if(file.is_open())
    {
        file >> root;
        file.close();
        return root;
    }
    return Json::Value();
}

typedef int (*args_fucntion)(Args*, char** argv);

int set_config_file(Args* arg, char** argv)
{
    arg->config_file=*argv;
    return 1;
}

int print_help(Args* arg, char** argv);

int _override_app_dir(Args* arg, char** argv){
    arg->override_app_dir=*argv;
    return 1;
}

int _override_pad_dir(Args* arg, char** argv){
    arg->override_pad_dir=*argv;
    return 1;
}



int _input_command(Args* args, char** argv)
{
    args->use_stdin=false;
    args->use_pipe=false;
    args->use_socket=false;
	std::string arg = std::string(*argv);
    if(arg=="stdin")
    {
        args->use_stdin=true;
        return 1;
    }
	else if(arg=="inet")
    {
		args->use_socket=true;
        args->use_socket_port=atoi(*(argv+1));
        return 2;
    }
	else if(arg=="unix")
    {
        args->use_pipe=true;
        args->use_pipe_filename=*(argv+1);
        return 2;
    }
    else{
        print_help(args, argv);
        exit(-1);
    }
}


int _keep_alive(Args* args, char** argv)
{
	args->keep_alive=1;
	return 0;
}

int _one_command(Args* args, char** argv)
{
	args->keep_alive=2;
	return 0;
}

const void* ARGS_FUNCTION_LIST[][5]={
    /* {LONG_OPTION; SHORT_OPTION, handler, param string for help, long help}*/
    {"--config", "-c", (void*)set_config_file, "CONFIG_FILE", "Utilise le fichier de configuration CONFIG_FILE"},
    {"--app-dir", "-d", (void*)_override_app_dir, "APP_DIR", "Change le dossier (remplace la config) de l'application avec APP_DIR"},
	{"--pads-dir", "-P", (void*)_override_pad_dir, "PAD_DIR", "Change le dossier (remplace la config) des pas avec PAD_DIR"},
	{"--keep-alive", "-k", (void*)_keep_alive, "", "Garde la connexion avec la socket ouverte après une commande"},
	{"--one-command", "-o", (void*)_one_command, "", "Ferme le socket après chaque commande"},
	{"--command", "-C", (void*)_input_command, "stdin | unix FILENAME | inet PORT", "La manière de lire les commandes. stdin: entrée standard (par défaut) \
, pipe FILENAME : va créer une pipe FILENAME, socket PORT: va lire sur lor port PORT"},
    {"--help", "-h", (void*)print_help, "", "Affiche cette aide"}
} ;



int print_help(Args* arg, char** argv)
{
    std::cerr << "simplemidi [OPTIONS] PAD_CONFIGURATION_FILE " << "\n";
    int l = sizeof (ARGS_FUNCTION_LIST)/sizeof (*ARGS_FUNCTION_LIST);
    std::cerr << "OPTIONS:\n";
    for(int i=0; i<l; i++)
    {
        std::cerr << "\t" << (const char*)ARGS_FUNCTION_LIST[i][0];
        if((const char*)ARGS_FUNCTION_LIST[i][1]) std::cerr << "|" << (const char*)ARGS_FUNCTION_LIST[i][1];
        std::cerr << " " <<(const char*)ARGS_FUNCTION_LIST[i][3] << " : \n\t\t" << (const char*)ARGS_FUNCTION_LIST[i][4] << "\n";
    }
    arg->continue_app=false;
    return 0;
}

Args::Args(int argc, char** argv)
{
    continue_app=true;
	keep_alive=0;
    use_stdin=true;
    use_pipe=use_socket=false;
    use_pipe_filename="";
    use_socket_port=-1;
    for(int i=1; i<argc; i++)
    {
        std::string arg = argv[i];
        int j, l = sizeof (ARGS_FUNCTION_LIST)/sizeof (*ARGS_FUNCTION_LIST);
        for(j=0; j<l; j++)
        {
            if(arg==(const char*)ARGS_FUNCTION_LIST[j][0] || arg==(const char*)ARGS_FUNCTION_LIST[j][1])
			{
                i+=((args_fucntion)ARGS_FUNCTION_LIST[j][2])(this, argv+i+1);
                break;
            }
        }
        if(j==l){
            pad_file=argv[i];
            return;
        }
    }
    if(pad_file==""){
        print_help(this, NULL);
    }
}

bool Fs::exists(const std::string& path)
{
    return is_file_exists(path, (~S_IFDIR)&0xffff);
}

bool Fs::is_file(const std::string& path)
{
    return is_file_exists(path, (~S_IFDIR)&0xff00);
}

bool Fs::is_dir(const std::string& path)
{
    return is_file_exists(path, S_IFDIR);
}

bool Fs::is_file_exists(const std::string& path, int mode)
{
    struct stat s;
    if(stat(path.c_str(), &s))
    {
        return false;
    }
    return s.st_mode&mode;
}

bool Fs::remove(const std::string& path)
{
    if(is_file(path))
        return !::remove(path.c_str());
    return false;
}
